@@ -28,6 +28,7 @@ class BaseEventLoopTests(test_utils.TestCase):
     def setUp(self):
         self.loop = base_events.BaseEventLoop()
         self.loop._selector = mock.Mock()
+        self.loop._selector.select.return_value = ()
         self.set_event_loop(self.loop)
 
     def test_not_implemented(self):
@@ -45,8 +46,6 @@ class BaseEventLoopTests(test_utils.TestCase):
             NotImplementedError, self.loop._process_events, [])
         self.assertRaises(
             NotImplementedError, self.loop._write_to_self)
-        self.assertRaises(
-            NotImplementedError, self.loop._read_from_self)
         self.assertRaises(
             NotImplementedError,
             self.loop._make_read_pipe_transport, m, m)
@@ -247,7 +246,8 @@ class BaseEventLoopTests(test_utils.TestCase):
     @mock.patch('trollius.base_events.logger')
     def test__run_once_logging(self, m_logger):
         def slow_select(timeout):
-            time.sleep(1.0)
+            # Sleep a bit longer than a second to avoid timer resolution issues.
+            time.sleep(1.1)
             return []
 
         # logging needs debug flag
@@ -288,6 +288,12 @@ class BaseEventLoopTests(test_utils.TestCase):
         self.assertRaises(TypeError,
             self.loop.run_until_complete, 'blah')
 
+    def test_run_until_complete_loop(self):
+        task = asyncio.Future(loop=self.loop)
+        other_loop = self.new_test_loop()
+        self.assertRaises(ValueError,
+            other_loop.run_until_complete, task)
+
     def test_subprocess_exec_invalid_args(self):
         args = [sys.executable, '-c', 'pass']
 
@@ -403,19 +409,22 @@ class BaseEventLoopTests(test_utils.TestCase):
             1/0
 
         def run_loop():
-            self.loop.call_soon(zero_error)
+            handle = self.loop.call_soon(zero_error)
             self.loop._run_once()
+            return handle
 
+        self.loop.set_debug(True)
         self.loop._process_events = mock.Mock()
 
         mock_handler = mock.Mock()
         self.loop.set_exception_handler(mock_handler)
-        run_loop()
+        handle = run_loop()
         mock_handler.assert_called_with(self.loop, {
             'exception': MOCK_ANY,
             'message': test_utils.MockPattern(
                                 'Exception in callback.*zero_error'),
-            'handle': MOCK_ANY,
+            'handle': handle,
+            'source_traceback': handle._source_traceback,
         })
         mock_handler.reset_mock()
 
@@ -511,6 +520,29 @@ class BaseEventLoopTests(test_utils.TestCase):
                                                TROLLIUSDEBUG='1')
         self.assertEqual(stdout.rstrip(), b'True')
 
+    def test_create_task(self):
+        class MyTask(asyncio.Task):
+            pass
+
+        @asyncio.coroutine
+        def test():
+            pass
+
+        class EventLoop(base_events.BaseEventLoop):
+            def create_task(self, coro):
+                return MyTask(coro, loop=loop)
+
+        loop = EventLoop()
+        self.set_event_loop(loop)
+
+        coro = test()
+        task = asyncio.async(coro, loop=loop)
+        self.assertIsInstance(task, MyTask)
+
+        # make warnings quiet
+        task._log_destroy_pending = False
+        coro.close()
+
 
 class MyProto(asyncio.Protocol):
     done = None
@@ -1028,7 +1060,7 @@ class BaseEventLoopWithSelectorTests(test_utils.TestCase):
         fmt = m_logger.warning.call_args[0][0]
         args = m_logger.warning.call_args[0][1:]
         self.assertRegex(fmt % tuple(args),
-                         "^Executing Handle.*stop_loop_cb.* took .* seconds$")
+                         "^Executing <Handle.*stop_loop_cb.*> took .* seconds$")
 
         # slow task
         asyncio.async(stop_loop_coro(self.loop), loop=self.loop)
@@ -1036,7 +1068,7 @@ class BaseEventLoopWithSelectorTests(test_utils.TestCase):
         fmt = m_logger.warning.call_args[0][0]
         args = m_logger.warning.call_args[0][1:]
         self.assertRegex(fmt % tuple(args),
-                         "^Executing Task.*stop_loop_coro.* took .* seconds$")
+                         "^Executing <Task.*stop_loop_coro.*> took .* seconds$")
 
 
 if __name__ == '__main__':
