@@ -1,16 +1,27 @@
+__all__ = ['coroutine',
+           'iscoroutinefunction', 'iscoroutine']
+
 import functools
 import inspect
+import opcode
 import os
 import sys
+import traceback
+import types
 try:
     import asyncio
 except ImportError:
     asyncio = None
 
 from . import compat
+from . import events
 from . import futures
 from .log import logger
 
+
+# Opcode of "yield from" instruction
+_YIELD_FROM = opcode.opmap.get('YIELD_FROM', None)
+
 # If you set _DEBUG to true, @coroutine will wrap the resulting
 # generator objects in a CoroWrapper instance (defined below).  That
 # instance will log a message when the generator is never iterated
@@ -22,7 +33,34 @@ from .log import logger
 # when _DEBUG is true.
 _DEBUG = bool(os.environ.get('TROLLIUSDEBUG'))
 
-_PY35 = (sys.version_info >= (3, 5))
+
+if _YIELD_FROM is not None:
+    # Check for CPython issue #21209
+    exec('''if 1:
+        def has_yield_from_bug():
+            class MyGen:
+                def __init__(self):
+                    self.send_args = None
+                def __iter__(self):
+                    return self
+                def __next__(self):
+                    return 42
+                def send(self, *what):
+                    self.send_args = what
+                    return None
+            def yield_from_gen(gen):
+                yield from gen
+            value = (1, 2, 3)
+            gen = MyGen()
+            coro = yield_from_gen(gen)
+            next(coro)
+            coro.send(value)
+            return gen.send_args != (value,)
+''')
+    _YIELD_FROM_BUG = has_yield_from_bug()
+    del has_yield_from_bug
+else:
+    _YIELD_FROM_BUG = False
 
 
 if compat.PY33:
@@ -59,12 +97,22 @@ else:
 
 
 class CoroWrapper(object):
-    # Wrapper for coroutine in _DEBUG mode.
+    # Wrapper for coroutine object in _DEBUG mode.
 
     def __init__(self, gen, func):
         assert inspect.isgenerator(gen), gen
         self.gen = gen
         self.func = func
+        self._source_traceback = traceback.extract_stack(sys._getframe(1))
+        # __name__, __qualname__, __doc__ attributes are set by the coroutine()
+        # decorator
+
+    def __repr__(self):
+        coro_repr = _format_coroutine(self)
+        if self._source_traceback:
+            frame = self._source_traceback[-1]
+            coro_repr += ', created at %s:%s' % (frame[0], frame[1])
+        return '<%s %s>' % (self.__class__.__name__, coro_repr)
 
     def __iter__(self):
         return self
@@ -73,13 +121,21 @@ class CoroWrapper(object):
         return next(self.gen)
     next = __next__
 
-    def send(self, *value):
-        # We use `*value` because of a bug in CPythons prior
-        # to 3.4.1. See issue #21209 and test_yield_from_corowrapper
-        # for details.  This workaround should be removed in 3.5.0.
-        if len(value) == 1:
-            value = value[0]
-        return self.gen.send(value)
+    if _YIELD_FROM_BUG:
+        # For for CPython issue #21209: using "yield from" and a custom
+        # generator, generator.send(tuple) unpacks the tuple instead of passing
+        # the tuple unchanged. Check if the caller is a generator using "yield
+        # from" to decide if the parameter should be unpacked or not.
+        def send(self, *value):
+            frame = sys._getframe()
+            caller = frame.f_back
+            assert caller.f_lasti >= 0
+            if caller.f_code.co_code[caller.f_lasti] != _YIELD_FROM:
+                value = value[0]
+            return self.gen.send(value)
+    else:
+        def send(self, value):
+            return self.gen.send(value)
 
     def throw(self, exc):
         return self.gen.throw(exc)
@@ -104,14 +160,14 @@ class CoroWrapper(object):
         gen = getattr(self, 'gen', None)
         frame = getattr(gen, 'gi_frame', None)
         if frame is not None and frame.f_lasti == -1:
-            func = self.func
-            code = func.__code__
-            filename = code.co_filename
-            lineno = code.co_firstlineno
-            logger.error(
-                'Coroutine %r defined at %s:%s was never yielded from',
-                func.__name__, filename, lineno)
-
+            msg = '%r was never yielded from' % self
+            tb = getattr(self, '_source_traceback', ())
+            if tb:
+                tb = ''.join(traceback.format_list(tb))
+                msg += ('\nCoroutine object created at '
+                        '(most recent call last):\n')
+                msg += tb.rstrip()
+            logger.error(msg)
 
 def coroutine(func):
     """Decorator to mark coroutines.
@@ -137,8 +193,10 @@ def coroutine(func):
         @functools.wraps(func)
         def wrapper(*args, **kwds):
             w = CoroWrapper(coro(*args, **kwds), func)
+            if w._source_traceback:
+                del w._source_traceback[-1]
             w.__name__ = func.__name__
-            if _PY35:
+            if hasattr(func, '__qualname__'):
                 w.__qualname__ = func.__qualname__
             w.__doc__ = func.__doc__
             return w
@@ -154,16 +212,41 @@ def iscoroutinefunction(func):
     return getattr(func, '_is_coroutine', False)
 
 
+_COROUTINE_TYPES = (types.GeneratorType, CoroWrapper)
 if asyncio is not None:
     # Accept also asyncio Future objects for interoperability
-    _COROUTINE_TYPES = (CoroWrapper, asyncio.tasks.CoroWrapper)
-else:
-    _COROUTINE_TYPES = CoroWrapper
-
+    if hasattr(asyncio, 'coroutines'):
+        _COROUTINE_TYPES += (asyncio.coroutines.CoroWrapper,)
+    else:
+        # old Tulip/Python versions
+        _COROUTINE_TYPES += (asyncio.tasks.CoroWrapper,)
 
 def iscoroutine(obj):
     """Return True if obj is a coroutine object."""
-    return isinstance(obj, _COROUTINE_TYPES) or inspect.isgenerator(obj)
+    return isinstance(obj, _COROUTINE_TYPES)
+
+
+def _format_coroutine(coro):
+    assert iscoroutine(coro)
+    coro_name = getattr(coro, '__qualname__', coro.__name__)
+
+    filename = coro.gi_code.co_filename
+    if (isinstance(coro, CoroWrapper)
+    and not inspect.isgeneratorfunction(coro.func)):
+        filename, lineno = events._get_function_source(coro.func)
+        if coro.gi_frame is None:
+            coro_repr = '%s() done, defined at %s:%s' % (coro_name, filename, lineno)
+        else:
+            coro_repr = '%s() running, defined at %s:%s' % (coro_name, filename, lineno)
+    elif coro.gi_frame is not None:
+        lineno = coro.gi_frame.f_lineno
+        coro_repr = '%s() running at %s:%s' % (coro_name, filename, lineno)
+    else:
+        lineno = coro.gi_code.co_firstlineno
+        coro_repr = '%s() done, defined at %s:%s' % (coro_name, filename, lineno)
+
+    return coro_repr
+
 
 class FromWrapper(object):
     __slots__ = ('obj',)
