@@ -12,6 +12,7 @@ __all__ = ['AbstractEventLoopPolicy',
 import functools
 import inspect
 import subprocess
+import traceback
 import threading
 import socket
 import sys
@@ -24,6 +25,7 @@ except ImportError:
 
 _PY34 = sys.version_info >= (3, 4)
 
+
 def _get_function_source(func):
     if _PY34:
         func = inspect.unwrap(func)
@@ -39,10 +41,43 @@ def _get_function_source(func):
     return None
 
 
+def _format_args(args):
+    # function formatting ('hello',) as ('hello')
+    args_repr = repr(args)
+    if len(args) == 1 and args_repr.endswith(',)'):
+        args_repr = args_repr[:-2] + ')'
+    return args_repr
+
+
+def _format_callback(func, args, suffix=''):
+    if isinstance(func, functools.partial):
+        if args is not None:
+            suffix = _format_args(args) + suffix
+        return _format_callback(func.func, func.args, suffix)
+
+    if compat.PY33:
+        func_repr = getattr(func, '__qualname__', None)
+    else:
+        func_repr = getattr(func, '__name__', None)
+    if not func_repr:
+        func_repr = repr(func)
+
+    if args is not None:
+        func_repr += _format_args(args)
+    if suffix:
+        func_repr += suffix
+
+    source = _get_function_source(func)
+    if source:
+        func_repr += ' at %s:%s' % source
+    return func_repr
+
+
 class Handle(object):
     """Object returned by callback registration methods."""
 
-    __slots__ = ['_callback', '_args', '_cancelled', '_loop', '__weakref__']
+    __slots__ = ('_callback', '_args', '_cancelled', '_loop',
+                 '_source_traceback', '__weakref__')
 
     def __init__(self, callback, args, loop):
         assert not isinstance(callback, Handle), 'A Handle is not a callback'
@@ -50,38 +85,41 @@ class Handle(object):
         self._callback = callback
         self._args = args
         self._cancelled = False
+        if self._loop.get_debug():
+            self._source_traceback = traceback.extract_stack(sys._getframe(1))
+        else:
+            self._source_traceback = None
 
     def __repr__(self):
-        if compat.PY33:
-            cb_repr = getattr(self._callback, '__qualname__', None)
-        else:
-            cb_repr = getattr(self._callback, '__name__', None)
-        if not cb_repr:
-            cb_repr = str(self._callback)
-
-        source = _get_function_source(self._callback)
-        if source:
-            cb_repr += ' at %s:%s' % source
-
-        res = 'Handle({0}, {1})'.format(cb_repr, self._args)
+        info = [self.__class__.__name__]
         if self._cancelled:
-            res += '<cancelled>'
-        return res
+            info.append('cancelled')
+        if self._callback is not None:
+            info.append(_format_callback(self._callback, self._args))
+        if self._source_traceback:
+            frame = self._source_traceback[-1]
+            info.append('created at %s:%s' % (frame[0], frame[1]))
+        return '<%s>' % ' '.join(info)
 
     def cancel(self):
         self._cancelled = True
+        self._callback = None
+        self._args = None
 
     def _run(self):
         try:
             self._callback(*self._args)
         except Exception as exc:
-            msg = 'Exception in callback {0}{1!r}'.format(self._callback,
-                                                          self._args)
-            self._loop.call_exception_handler({
+            cb = _format_callback(self._callback, self._args)
+            msg = 'Exception in callback {0}'.format(cb)
+            context = {
                 'message': msg,
                 'exception': exc,
                 'handle': self,
-            })
+            }
+            if self._source_traceback:
+                context['source_traceback'] = self._source_traceback
+            self._loop.call_exception_handler(context)
         self = None  # Needed to break cycles when an exception occurs.
 
 
@@ -93,17 +131,21 @@ class TimerHandle(Handle):
     def __init__(self, when, callback, args, loop):
         assert when is not None
         super(TimerHandle, self).__init__(callback, args, loop)
-
+        if self._source_traceback:
+            del self._source_traceback[-1]
         self._when = when
 
     def __repr__(self):
-        res = 'TimerHandle({0}, {1}, {2})'.format(self._when,
-                                                  self._callback,
-                                                  self._args)
+        info = []
         if self._cancelled:
-            res += '<cancelled>'
-
-        return res
+            info.append('cancelled')
+        info.append('when=%s' % self._when)
+        if self._callback is not None:
+            info.append(_format_callback(self._callback, self._args))
+        if self._source_traceback:
+            frame = self._source_traceback[-1]
+            info.append('created at %s:%s' % (frame[0], frame[1]))
+        return '<%s %s>' % (self.__class__.__name__, ' '.join(info))
 
     def __hash__(self):
         return hash(self._when)
@@ -184,6 +226,10 @@ else:
             """Return whether the event loop is currently running."""
             raise NotImplementedError
 
+        def is_closed(self):
+            """Returns True if the event loop was closed."""
+            raise NotImplementedError
+
         def close(self):
             """Close the loop.
 
@@ -209,6 +255,11 @@ else:
         def time(self):
             raise NotImplementedError
 
+        # Method scheduling a coroutine object: create a task.
+
+        def create_task(self, coro):
+            raise NotImplementedError
+
         # Methods for interacting with threads.
 
         def call_soon_threadsafe(self, callback, *args):
