@@ -1,8 +1,7 @@
 """Support for tasks, coroutines and the scheduler."""
 from __future__ import print_function
 
-__all__ = ['coroutine', 'Task',
-           'iscoroutinefunction', 'iscoroutine',
+__all__ = ['Task',
            'FIRST_COMPLETED', 'FIRST_EXCEPTION', 'ALL_COMPLETED',
            'wait', 'wait_for', 'as_completed', 'sleep', 'async',
            'gather', 'shield', 'Return', 'From',
@@ -10,6 +9,7 @@ __all__ = ['coroutine', 'Task',
 
 import functools
 import linecache
+import sys
 import traceback
 try:
     from weakref import WeakSet
@@ -22,12 +22,12 @@ except ImportError:
     asyncio = None
 
 from . import compat
+from . import coroutines
 from . import events
 from . import executor
 from . import futures
 from .locks import Lock, Condition, Semaphore, _ContextManager
-from .coroutines import Return, From, coroutine, iscoroutinefunction, iscoroutine
-from . import coroutines
+from .coroutines import coroutine, From, Return, iscoroutinefunction, iscoroutine
 
 
 if asyncio is not None:
@@ -36,6 +36,8 @@ if asyncio is not None:
 else:
     _FUTURE_CLASSES = futures.Future
 
+_PY34 = (sys.version_info >= (3, 4))
+
 
 @coroutine
 def _lock_coroutine(lock):
@@ -85,33 +87,58 @@ class Task(futures.Future):
         return set(t for t in cls._all_tasks if t._loop is loop)
 
     def __init__(self, coro, loop=None):
-        assert iscoroutine(coro), repr(coro)  # Not a coroutine function!
+        assert coroutines.iscoroutine(coro), repr(coro)  # Not a coroutine function!
         super(Task, self).__init__(loop=loop)
+        if self._source_traceback:
+            del self._source_traceback[-1]
         self._coro = iter(coro)  # Use the iterator just in case.
         self._fut_waiter = None
         self._must_cancel = False
         self._loop.call_soon(self._step)
         self.__class__._all_tasks.add(self)
+        # If False, don't log a message if the task is destroyed whereas its
+        # status is still pending
+        self._log_destroy_pending = True
+
+    # On Python 3.3 or older, objects with a destructor part of a reference
+    # cycle are never destroyed. It's not more the case on Python 3.4 thanks to
+    # the PEP 442.
+    if _PY34:
+        def __del__(self):
+            if self._state == futures._PENDING and self._log_destroy_pending:
+                context = {
+                    'task': self,
+                    'message': 'Task was destroyed but it is pending!',
+                }
+                if self._source_traceback:
+                    context['source_traceback'] = self._source_traceback
+                self._loop.call_exception_handler(context)
+            futures.Future.__del__(self)
 
     def __repr__(self):
-        res = super(Task, self).__repr__()
-        if (self._must_cancel and
-            self._state == futures._PENDING and
-            '<PENDING' in res):
-            res = res.replace('<PENDING', '<CANCELLING', 1)
-        i = res.find('<')
-        if i < 0:
-            i = len(res)
-        text = self._coro.__name__
-        coro = self._coro
-        if iscoroutine(coro):
-            filename = coro.gi_code.co_filename
-            if coro.gi_frame is not None:
-                text += ' at %s:%s' % (filename, coro.gi_frame.f_lineno)
-            else:
-                text += ' done at %s' % filename
-        res = res[:i] + '(<{0}>)'.format(text) + res[i:]
-        return res
+        info = []
+        if self._must_cancel:
+            info.append('cancelling')
+        else:
+            info.append(self._state.lower())
+
+        coro = coroutines._format_coroutine(self._coro)
+        info.append('coro=<%s>' % coro)
+
+        if self._source_traceback:
+            frame = self._source_traceback[-1]
+            info.append('created at %s:%s' % (frame[0], frame[1]))
+
+        if self._state == futures._FINISHED:
+            info.append(self._format_result())
+
+        if self._callbacks:
+            info.append(self._format_callbacks())
+
+        if self._fut_waiter is not None:
+            info.append('wait_for=%r' % self._fut_waiter)
+
+        return '<%s %s>' % (self.__class__.__name__, ' '.join(info))
 
     def get_stack(self, limit=None):
         """Return the list of stack frames for this task's coroutine.
@@ -276,7 +303,7 @@ class Task(futures.Future):
             # FIXME: faster check. common base class? hasattr?
             elif isinstance(result, (Lock, Condition, Semaphore)):
                 coro = _lock_coroutine(result)
-                result = Task(coro, loop=self._loop)
+                result = self._loop.create_task(coro)
 
             if isinstance(result, _FUTURE_CLASSES):
                 # Yielded Future must come from Future.__iter__().
@@ -334,18 +361,18 @@ def wait(fs, loop=None, timeout=None, return_when=ALL_COMPLETED):
     Note: This does not raise TimeoutError! Futures that aren't done
     when the timeout occurs are returned in the second set.
     """
-    if isinstance(fs, futures.Future) or iscoroutine(fs):
+    if isinstance(fs, futures.Future) or coroutines.iscoroutine(fs):
         raise TypeError("expect a list of futures, not %s" % type(fs).__name__)
     if not fs:
         raise ValueError('Set of coroutines/Futures is empty.')
+    if return_when not in (FIRST_COMPLETED, FIRST_EXCEPTION, ALL_COMPLETED):
+        raise ValueError('Invalid return_when value: {0}'.format(return_when))
 
     if loop is None:
         loop = events.get_event_loop()
 
     fs = set(async(f, loop=loop) for f in set(fs))
 
-    if return_when not in (FIRST_COMPLETED, FIRST_EXCEPTION, ALL_COMPLETED):
-        raise ValueError('Invalid return_when value: {0}'.format(return_when))
     result = yield From(_wait(fs, timeout, return_when, loop))
     raise Return(result)
 
@@ -456,7 +483,7 @@ def as_completed(fs, loop=None, timeout=None):
 
     Note: The futures 'f' are not necessarily members of fs.
     """
-    if isinstance(fs, futures.Future) or iscoroutine(fs):
+    if isinstance(fs, futures.Future) or coroutines.iscoroutine(fs):
         raise TypeError("expect a list of futures, not %s" % type(fs).__name__)
     loop = loop if loop is not None else events.get_event_loop()
     todo = set(async(f, loop=loop) for f in set(fs))
@@ -498,7 +525,8 @@ def as_completed(fs, loop=None, timeout=None):
 def sleep(delay, result=None, loop=None):
     """Coroutine that completes after a given time (in seconds)."""
     future = futures.Future(loop=loop)
-    h = future._loop.call_later(delay, future.set_result, result)
+    h = future._loop.call_later(delay,
+                                future._set_result_unless_cancelled, result)
     try:
         result = yield From(future)
         raise Return(result)
@@ -518,8 +546,13 @@ def async(coro_or_future, loop=None):
         if loop is not None and loop is not coro_or_future._loop:
             raise ValueError('loop argument must agree with Future')
         return coro_or_future
-    elif iscoroutine(coro_or_future):
-        return Task(coro_or_future, loop=loop)
+    elif coroutines.iscoroutine(coro_or_future):
+        if loop is None:
+            loop = events.get_event_loop()
+        task = loop.create_task(coro_or_future)
+        if task._source_traceback:
+            del task._source_traceback[-1]
+        return task
     else:
         raise TypeError('A Future or coroutine is required')
 
@@ -569,22 +602,33 @@ def gather(*coros_or_futures, **kw):
     if kw:
         raise TypeError("unexpected keyword")
 
-    arg_to_fut = dict((arg, async(arg, loop=loop))
-                      for arg in set(coros_or_futures))
-    children = [arg_to_fut[arg] for arg in coros_or_futures]
-    n = len(children)
-    if n == 0:
+    if not coros_or_futures:
         outer = futures.Future(loop=loop)
         outer.set_result([])
         return outer
-    if loop is None:
-        loop = children[0]._loop
-    for fut in children:
-        if fut._loop is not loop:
-            raise ValueError("futures are tied to different event loops")
+
+    arg_to_fut = {}
+    for arg in set(coros_or_futures):
+        if not isinstance(arg, futures.Future):
+            fut = async(arg, loop=loop)
+            if loop is None:
+                loop = fut._loop
+            # The caller cannot control this future, the "destroy pending task"
+            # warning should not be emitted.
+            fut._log_destroy_pending = False
+        else:
+            fut = arg
+            if loop is None:
+                loop = fut._loop
+            elif fut._loop is not loop:
+                raise ValueError("futures are tied to different event loops")
+        arg_to_fut[arg] = fut
+
+    children = [arg_to_fut[arg] for arg in coros_or_futures]
+    nchildren = len(children)
     outer = _GatheringFuture(children, loop=loop)
     non_local = {'nfinished': 0}
-    results = [None] * n
+    results = [None] * nchildren
 
     def _done_callback(i, fut):
         if outer._state != futures._PENDING:
@@ -606,7 +650,7 @@ def gather(*coros_or_futures, **kw):
             res = fut._result
         results[i] = res
         non_local['nfinished'] += 1
-        if non_local['nfinished'] == n:
+        if non_local['nfinished'] == nchildren:
             outer.set_result(results)
 
     for i, fut in enumerate(children):
